function [Bg, Bo, rhoo, rhog, Rs ] = FVF (go, gg, T, p)
% Created by Harald A. Asheim and modified to the spesific situation
% Calculations of formation volume factors
% p = Pressure at position i in the tubing
% T = Temperature at position i in the tubing
% go = Oil specific gravity at surface conditions
% gg = Gas specific gravity at surface conditions

rhg0=gg*1.23;       % kg/Sm3  Gas density at surface conditions
rho0=1000*go;       % kg/Sm3  Oil density at surface conditions

% Pseudocritical Temperature and Pressure
Tpc =94.0+194.2*gg-41.1*gg.^2;          
Ppc=1e5*(52.18-9.032*gg-0.248*gg.^2);
% Reduced Temperature and Pressure:
Tr=T/Tpc;
pR=p/Ppc;

[ z ] = zfak( pR,Tr,0.8);   % Yarborough and Hall (1974) EOS

% Calculating gas and oil FVF at position i
Bg =1.01e5/p*T/(273+15)*z;
Rs =0.0059*gg*10^(2.14/go-0.00198*T)*(0.797*10^-5*p+1.4)^1.205;
Bo =0.9759+0.000952*((gg/go)^0.5*Rs+0.401*T-103)^1.2;
% Calculating gas and oil density at position i
rhog=rhg0/Bg;
rhoo=(rho0+Rs*gg*1.22)/Bo;
