clear
PeriodStableJan;       % Considered period
WellSpecifications     % From given well specifications

%Gas distribution factor depends on the deviation in the rate
if sigma < 600
    Co=1.30;
elseif sigma > 600 && sigma < 850
    Co=1.36;
elseif sigma > 850 && sigma < 1000
    Co=1.43;
elseif sigma > 1000 && sigma < 1300
    Co=1.55;
else 
    Co=1.60;
end
    
myg= 1e-5;              % Pa*s  Viscosity of gas guess
Twh= mean(twh);         % C     Wellhead temperature
Tgrad=(Tres-Twh)/Lres;  % C     Geothermal gradient

%------------Preallocating-----------------
k= 1;                   %       k=i-1, the previous value
Preallocating
wor=mean(qw)/(mean(qw+qo));
%---Numerical Integration of the pressure along annulus--
% At the wellhead:                           
Pg(1)=mean(pgi)*10^5;
T=Twh+Tgrad*1+274.15;
[Bga(1),rhoga(1)]=FVFannulus(gg,T,Pg(1));
Qga(1)=Qg_i*Bga(1);                     
% Along the rest of the annulus:
for i=2:Lres    % annulus
    T=Twh+Tgrad*i+274.15;           % K     Temperature at i
    [Bga(i),rhoga(i), z]=FVFannulus(gg,T,Pg(k));%Annular gas FVF
    Qga(i)=Qg_i*Bga(i);             % m3/s  Gas rate in annulus
    if i<=Linj                      %       Above the gas valve
        Pg(i)=Pg(k)+g*rhoga(i);     % Pa    Gas pressure in annulus
    else                            %       Below the gas valve
        Pg(i)=Pg(k)+g*rhow;         % Pa    Pressure in annulus
    end
     Qg_inj=Qga(Linj);              %       Lift gas rate at Linj
     k=k+1;                         %       Counter
end
k=1;                                %       Reset counter

%----Numerical Integration of the pressure along tubing------
% At the tubing head:
p(1)=pth;
T=Twh+Tgrad*1+274.15;                
[Bg(1), Bo(1), rhoo, rhog, Rs(1)] = FVF (go, gg, T, p(1));
Qg_tbg(1)=Qg_r*Bg(1);
yl(1)= 0.145;
yg(1)=1-yl(1);
vsg(1)=Qg_tbg(1)/At; 
vsl(1)=mean(qw+qo*Bo(1))/(24*60*60)/At; 
% In the rest of the tubing:
for i=2:Lres    % tubing
    T=Twh+Tgrad*i+274.15;           % K     Temperature at i
    myo=6e-8*(p(k)/10^5)^2-3.01e-5*p(k)/10^5+6.3348e-3;% Pa*s  Oil viscosity
    [Bg(i),Bo(i),rhoo,rhog,Rs(i)]=FVF(go,gg,T,p(k));% FVF
    Qo(i)=mean(qo)/(24*60*60)*Bo(i);% m3/s  Oil rate
    Ql(i)=mean(qw)/(24*60*60)+Qo(i);% m3/s  Liquid rate 
    if i<=Linj                      %       Above gas valve
        Qg_tbg(i)=Qg_r*Bg(i);       % m3/s  Gas rate in tubing
    else                            %       Below gas valve
        Qg_tbg(i)=Qg_r*Bg(i)-Qg_inj;% m3/s  Gas rate in tubing
    end
    rhol=rhoo*(1-wor)+rhow*wor;     % kg/m3 Density of oil+water
    vsg(i)=Qg_tbg(i)/At;            % m/s   Gas superficial v.
    vsl(i)=Ql(i)/At;                % m/s   Liquid superficial v.
    vm(i)=vsl(i)+vsg(i);            % m/s   Mixture velocity 
    vc(i)=Co*vm(i);                 % m/s   Kinematic velocity
    yl(i)=0.5*sqrt((vsg(i)/vo+Co*vsl(i)/vo-1)^2+4*Co*vsl(i)/vo)-0.5*(vsg(i)/vo+Co*vsl(i)/vo-1);%   Liquid fraction
    yg(i)=1-yl(i);                  %       Gas fraction
    rhotp(i)=rhol*yl(i)+rhog*yg(i); % kg/m3 Density of two-phase
    myw=2.414e-5*10^(247.8/(T-140));% Pa*s  Water viscosity
    myl=myo*(1-wor)+myw*wor;        % Pa*s  Liquid viscosity
    my=myl*yl(i)+myg*yg(i);         % Pa*s  Mixture viscosity
    Re=rhotp(i)*vm(i)*d/my;         %       Reynolds number
    f(i)= Af*Re^(-Bf);              %       Friction factor
    p(i)=p(k)+g*rhotp(i)+0.5*f(i)*rhotp(i)/d*vm(i)^2;%Pa Pressure
    k=k+1;                          %       Counter 
end
k=1;                                %       Reset counter

%---Compare Measured Pw (MFM) and the calculated p(Lmfm)----
pw=mean(pwf);                       % Pa    Wellbore pressure
err=(pw-p(Lmfm)./10^5);             % Bar   Diff pressure

Vtry=[Co, Af, Bf, err, p(Lmfm)./10^5, Ql(Lmfm)*24*60*60, (err/pw)*100]
%---Plot of the well pressure and annulus pressure vs depth----
maxP=200;
InjDepth=-Linj*ones(1,maxP);        % L      Gas valve depth
MFMDepth=-Lmfm*ones(1,maxP);        % L      Gauge carrier depth
ResDepth=-Lres*ones(1,maxP);        % L      Casing depth
xp=[1:maxP]-1;
figure(1)
plot(xp,InjDepth,'--r',xp,MFMDepth,'--g',xp,ResDepth,'--m',p(1:Lres)./10^5,-(1:Lres),'b',Pg(1:Lres)./10^5,-(1:Lres),'r')
axis([0 200, -3000 0])
legend('Injection Depth','Measurement Depth', 'Reservoir Depth');
title('Pressure Profile');
xlabel('\bf Pressure [bar]');
ylabel('\bf Depth [m]');
grid
%------Plot of the superficial velocities vs depth--------
maxV=10;
InjDepth=-Linj*ones(1,maxV);        % L      Gas valve depth
MFMDepth=-Lmfm*ones(1,maxV);        % L      Gauge carrier depth
ResDepth=-Lres*ones(1,maxV);        % L      Casing depth
xv=[1:maxV]-1;
figure(2)
plot(-(1:Lres),vsg,-(1:Lres),vsl,InjDepth,xv,'--r',MFMDepth,xv,'--g',ResDepth,xv,'--m')
legend('Gas','Liquid');
title('Velocity Profile');
xlabel('\bf Depth [m]');
ylabel('\bf Velocity [m/s]');
grid
%---------Plot of the liquid fraction vs depth-------
maxV=2;
InjDepth=-Linj*ones(1,maxV);        % L      Gas valve depth
MFMDepth=-Lmfm*ones(1,maxV);        % L      Gauge carrier depth
ResDepth=-Lres*ones(1,maxV);        % L      Casing depth
xv=[1:maxV]-1;
figure(3)
plot(xv,InjDepth,'--r',xv,MFMDepth,'--g',xv,ResDepth,'--m',yg,-(1:Lres),'b')
legend('Injection Depth','Measurement Depth', 'Reservoir Depth');
title('Liquid Fraction vs Depth');
xlabel('\bf Liquid fraction');
ylabel('\bf Depth [m]');
grid
